package lrs.visitor;
import lrs.*;

/**
 * Merges the sorted input list into the sorted host list.
 * In the end, host and the input list share the same nodes and are sorted.
 * @author Dung X. Nguyen - Copyright 2002 - All rights reserved.
 */
public class Merge implements IAlgo {
    public final static Merge Singleton = new Merge();
    private Merge() {
    }

    /**
     * @param host is empty.
     * @param input an LRStruct
     * @return null
     */
    public Object emptyCase(LRStruct host, Object input) {
        return host.execute (Becomes.Singleton, input);
    }

    /**
     * @param host is not empty
     * @param input an LRStruct
     * @return null
     */
    public Object nonEmptyCase(LRStruct host, Object input) {
        return ((LRStruct)input).execute (new IAlgo() {
            public Object emptyCase(LRStruct h, Object inp) {
                return h.execute (Becomes.Singleton, inp);
            }

            /**
             * @param host a non-empty LRStruct
             * @param input a non-empty LRStruct.
             */
            public Object nonEmptyCase(LRStruct h, Object inp) {
                LRStruct inList = (LRStruct)inp;
                int hFirst = ((Integer)h.getFirst ()).intValue ();
                int inpFirst = ((Integer)inList.getFirst ()).intValue ();
                if (hFirst <= inpFirst) {
                    h.getRest ().execute(this, inList);
                    return inList.execute(Becomes.Singleton, h);
                }
                else {
                    inList.getRest ().execute (this, h);
                    return h.execute(Becomes.Singleton, inList);
                }
            }
        }, host);
    }

    /**
     * Simple test code.
     */
    public static void main(String[] args) {
        LRStruct l1 = new LRStruct ();
        l1.insertFront (new Integer (263));
        l1.insertFront (new Integer (15));
        l1.insertFront (new Integer (-9));
        l1.insertFront (new Integer (-72));
        System.out.print ("l1: " + l1);
        System.out.println();
        LRStruct l2 = new LRStruct ();

        /*
        //Testing base case.
        System.out.println ("Merging l1 into l2...");
        l2.execute (Merge.Singleton, l1);
        System.out.println ("l1: " + l1);
        System.out.println ("l2: " + l2);
        */

        l2.insertFront (new Integer (48));
        l2.insertFront (new Integer (0));
        l2.insertFront (new Integer (-2));
        System.out.print ("l2: " + l2);
        System.out.println();
        System.out.println ("Merging l1, l2...");
        l1.execute (Merge.Singleton, l2);
        System.out.println ("l1: " + l1);
        System.out.println ("l2: " + l2);
        System.out.println();
        try {
             System.out.println ("Press Enter to quit...");
             System.in.read ();
        }
        catch (Exception e) {
        }
    }
}

