package lrs.visitor;
import lrs.*;

/**
 * Merges the sorted input list into the sorted host list.
 * In the end, host and the input list share the same nodes and are sorted.
 * @author Dung X. Nguyen - Copyright 2002 - All rights reserved.
 */
public class Unzip implements IAlgo {
    public final static Unzip Singleton = new Unzip();
    private Unzip() {
    }

    /**
     * @param host is empty.
     * @param nu not used
     * @return an empty LRStruct.
     */
    public Object emptyCase(LRStruct host, Object nu) {
        return new LRStruct();
    }

    /**
     * @param host is not empty
     * @param nu not used
     * @return LRStruct containing the odd indexed element of the orignal host.
     */
    public Object nonEmptyCase(LRStruct host, Object nu) {
        // Hold on to the tail before setting the host's rest to a different list:
        LRStruct tail = host.getRest();
        // Get the odd indexed elements of the tail:
        LRStruct other = (LRStruct)tail.execute(this, null);
        // The odd indexed elements of the tail are the even-indexed elements
        // of the original host:
        host.setRest(other);
        return tail;
    }

    /**
     * Simple test code.
     */
    public static void main(String[] args) {
        LRStruct l1 = new LRStruct ();
        System.out.println("Before unzipping l1 = " + l1);
        System.out.println ("Unzipping l1 returns: " + l1.execute(Unzip.Singleton, null));
        System.out.println("After unzipping l1 = " + l1);
        System.out.println();
        l1.insertFront (new Integer (263));
        System.out.println("Before unzipping l1 = " + l1);
        System.out.println ("Unzipping l1 returns: " + l1.execute(Unzip.Singleton, null));
        System.out.println("After unzipping l1 = " + l1);
        System.out.println();
        l1.insertFront (new Integer (15));
        System.out.println("Before unzipping l1 = " + l1);
        System.out.println ("Unzipping l1 returns: " + l1.execute(Unzip.Singleton, null));
        System.out.println("After unzipping l1 = " + l1);
        System.out.println();
        l1.insertFront (new Integer (-9));
        l1.insertFront (new Integer (-72));
        System.out.println("Before unzipping l1 = " + l1);
        System.out.println ("Unzipping l1 returns: " + l1.execute(Unzip.Singleton, null));
        System.out.println("After unzipping l1 = " + l1);
        System.out.println();
        l1.insertFront (new Integer (0));
        l1.insertFront (new Integer (55));
        System.out.println("Before unzipping l1 = " + l1);
        System.out.println ("Unzipping l1 returns: " + l1.execute(Unzip.Singleton, null));
        System.out.println("After unzipping l1 = " + l1);
        System.out.println();
    }
}

